package convertisseureuroboa2poa;

import convertisseureuroboa2poa.Convertisseur.*;
import org.omg.CosNaming.*;
import org.omg.PortableServer.*;
import com.inprise.vbroker.orb.*;
import com.inprise.vbroker.PortableServerExt.BindSupportPolicyValue;
import com.inprise.vbroker.PortableServerExt.BindSupportPolicyValueHelper;


/**
 * Title:        Convertisseur Euros-Francs avec le POA
 * Description:  Illustration du passage du BOA vers le POA
 * Le POA est un adaptateur d'objets portable c'est  dire que
 * son fonctionnement sera identique sur diffrentes implmntations de CORBA
 * (car il est mieux spcifier que la BOA).
 * De plus, la POA introduit (ou modifie) les notions de :
 * - servant (entit qui implente une interface IDL)
 * - serveur (entit qui loge un ensemble de servants)
 * - rfrences d'objets (vue cliente d'une interface IDL)
 * -
 * Copyright:    Copyright (c) 2001
 * Company:      MIAGe
 * @author DR
 * @version 1.0
 */

public class ServeurConvertisseurEuro {

  public static void main(String[] args) {
    try {
      //intialisation de l'orb
      org.omg.CORBA.ORB orb = com.inprise.vbroker.orb.ORB.init(args,null);

      //rfrences initiales
      String [] ids = orb.list_initial_services();
      System.out.println("*** Liste des rfrences disponibles :");
      for(int i=0;i<ids.length;i++)
        System.out.println(ids[i]);
      System.out.println("***");

      /**
       *  On ne doit plus initialiser le BOA ...
       * com.inprise.vbroker.CORBA.BOA boa = ((com.inprise.vbroker.CORBA.ORB)orb).BOA_init();
       * ... mais on rcupre le rootPOA.
       * Le rootPOA est le seul POA qui existe par dfaut sur le serveur.
       * On pourra ensuite crer des POA fils dont on paramtrera les politiques
       * pour qu'ils grent un ensemble de servants comme on le dsire.
       */
      POA rootPOA = POAHelper.narrow(orb.resolve_initial_references("RootPOA"));

      /**
       *  On va crer des regles ou politiques (policies) pour utiliser avec
       *  un POA personnel.
       *  Pour notre POA, on choisit la valeur PERSISTENT pour la rgle Lifespan
       *  Cette rgle permet de rendre les rfrences d'objets persistantes (si
       *  on relance le serveur, le servant aura la meme reference).
       *  Le rootPOA ne supporte que des objets TRANSIENT alors que l'on
       * a besoin d'objets PERSISTENT donc on va crer un POA avec
       * une politique persistante ...
       * ... Cration de politiques pour un POA PERSISTENT
       */
      org.omg.CORBA.Any any = orb.create_any();
      BindSupportPolicyValueHelper.insert(any, BindSupportPolicyValue.BY_INSTANCE);
      org.omg.CORBA.Policy bsPolicy = orb.create_policy(com.inprise.vbroker.PortableServerExt.BIND_SUPPORT_POLICY_TYPE.value, any);
      org.omg.CORBA.Policy[] policies = {rootPOA.create_lifespan_policy(LifespanPolicyValue.PERSISTENT),bsPolicy};

      /**
       * Cration de son propre POA avec les politiques prcdentes et avec
       * le meme POA Manager que son pere, en l'occurrence le RootPOA.
       */
      POA poa = rootPOA.create_POA("monPOA",rootPOA.the_POAManager(),policies);

      /**
       *  Avec un BOA, un servant est un objet CORBA.
       *  Avec un POA, un servant et un objet sont distincts.
       *  Ainsi, on dissosie la notion d'objet d'implantation servant)
       *  de la notion de rf. d'obj. A prsent, on peut asocier plusieurs
       *  rf. d'obj. au mme servant. Les lcients pour leurs parts croiront
       *  s'adresser  des obj distincts.
       *
       *
       *  Au lieu de crer un objet CORBA et d'appeler obj_is_ready
       *  pour l'activer, on cre un servant et on active ce servant
       *  avec un ID sur notre POA. Si l'on ne veut pas donner son propre ID
       *  au servant, on peut ecrire directement :
       *  byte[] monEuroId = null;
       *  monEuroId = rootPOA.activate_object(moneuro);
       *
       *  ... Cration du servant
       */
      EuroImpl monEuro = new EuroImpl();

      /**
       * Donner un Object ID en convertissant "Euro" en un tableau d'octets
       */
      byte[] monEuroId = "Euro".getBytes();

      /**
       * Activer le servant avec l'ID dans le POA
       */
      poa.activate_object_with_id(monEuroId, monEuro);

      /**
       * Cette tape n'a pas d'quivalent avec le BOA.
       * Le root POA et notre POA personnel doivent tre manags
       * par un gestionnaire de POA, le POA manager. Ce gestionnaire peut
       * grer simultanment plusieurs POA dont il controle
       * l'tat de traitement (empilation des requetes, desactivation, ...).
       *
       * ... Activer le POA manager (passage de l'tat Inactif  l'tat Actif)
       */
      rootPOA.the_POAManager().activate();

      /**
       * On affiche un trace  l'cran de la rfrence d'objet
       * associe au servant monEuro. On aurait pu utilser la mthode
       * poa.id_to_reference(monEuroId) pour obtenir le meme resultat.
       */
      System.out.println(poa.servant_to_reference(monEuro) + " est pret.");


      /**
       * On n'utilise plus boa.impl_is_ready(); mais orb.run()
       * pour attendre les requetes ...
       *
       * ...Mise en attente de requete
       */
      orb.run();
    }
    catch (Exception e) {
      e.printStackTrace();
    }
  } //fin du main

} //fin de la classe